"""This file is a very simple example of how to use Textual CSS to
set the sizes and borders of widgets.

Recipe by Edward Jazzhands"""

import sys
from textual import on
from textual.app import App
from textual.widgets import Button, RichLog, Footer
from textual.containers import Container


class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    RichLog { 
        border: solid blue; 
        width: 70%;
        height: 1fr;
    }
    #button_container {
        border: solid green;
        height: 15;
    }
    """

    BINDINGS = [
        ("g", "send_log_msg", "Send a Log Message"),
    ]

    def compose(self):

        yield RichLog()
        with Container(id="button_container"):
            yield Button("Click to send log msg", id="my_button")
        yield Footer()

    @on(Button.Pressed, "#my_button")
    def action_send_log_msg(self):
        self.query_one(RichLog).write("This is a log message")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates how the `exclusive` parameter of the `work`
decorator can only be used to cancel/debounce workers that are normal
async workers (non-threaded). The normal worker will be restarted
each time the button is pressed and effectively cancel the currently
running worker. But threaded workers will not be cancelled
and will run to completion, even if the button is pressed multiple
times (This is a limitation of threading in Python and not because of Textual).

Recipe by NSPC911  
https://github.com/NSPC911"""


from asyncio import sleep

import sys
from textual import work, on
from textual.app import App, ComposeResult
from textual.widgets import Button, RichLog
from textual.containers import HorizontalGroup

class TextualApp(App[None]):
    def compose(self) -> ComposeResult:
        self.richlog = RichLog()
        yield self.richlog
        with HorizontalGroup():
            yield Button("Run worker", id="worker")
            yield Button("Run worker THREAD", id="thread")

    def on_button_pressed(self, event:Button.Pressed):
        self.richlog.write(event.button.id)

    @on(Button.Pressed, "#worker")
    @work(exclusive=True, thread=False)
    async def worker_runner(self, event:Button.Pressed):
        await sleep(1) # simulate process intensive thing
        self.richlog.write("Worker completed!")

    @on(Button.Pressed, "#thread")
    @work(exclusive=True, thread=True)
    def thread_runner(self, event:Button.Pressed):
        self.call_from_thread(sleep, 1)
        self.richlog.write("Thread completed!")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates ?

Recipe by Edward Jazzhands"""


import sys
from textual.app import App, ComposeResult
from textual.containers import Horizontal
from textual.reactive import reactive
from textual.widgets import Button, RichLog


class TextualApp(App[None]):
    CSS = """
    #buttons {
        dock: top;
        height: auto;
    }
    """

    count = reactive(0, always_update=True)

    def validate_count(self, count: int) -> int:
        """Validate value."""
        self.log(f"ValidatING {count=}")

        if count < 0:
            self.log("Count cannot be negative")
            count = 0
        elif count > 10:
            self.log("Count cannot be greater than 10")
            count = 10

        self.log(f"Validated {count=}")
        return count

    def watch_count(self, count: int) -> None:
        """Watch for changes to count."""
        self.log(f"Count changed to {count=}")

    def compose(self) -> ComposeResult:
        yield Horizontal(
            Button("+1", id="plus", variant="success"),
            Button("-1", id="minus", variant="error"),
            id="buttons",
        )
        yield RichLog(highlight=True)

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "plus":
            self.count += 1
        else:
            self.count -= 1
        self.query_one(RichLog).write(f"count = {self.count}")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how to use the configparser module in a Textual application.

Recipe by Edward Jazzhands"""

import sys
import configparser
from pathlib import Path
from textual.app import App

class TextualApp(App[None]):
    
    def __init__(self):
        super().__init__()

        config_path = Path(__file__).resolve().parent / "config.ini"
        if not config_path.exists():
            raise FileNotFoundError("config.ini file not found.")

        self.config = configparser.ConfigParser()  # Available globally as self.app.config
        self.config.read(config_path)

        ## Config settings ##
        self.my_string  = self.config.get("MAIN", "my_string")
        self.my_boolean = self.config.getboolean("MAIN", "my_boolean")
        self.my_integer = self.config.getint("MAIN", "my_integer")
        self.my_float   = self.config.getfloat("MAIN", "my_float")

    def on_mount(self) -> None:

        assert self.my_string == "Hello, World!"
        assert self.my_boolean is True
        assert self.my_integer == 42
        assert self.my_float == 3.14

    def on_ready(self) -> None:
        self.log("Configuration loaded successfully!")
        self.log(f"String: {self.my_string}")
        self.log(f"Boolean: {self.my_boolean}")
        self.log(f"Integer: {self.my_integer}")
        self.log(f"Float: {self.my_float}")
        


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""A very simple demonstration of colorirzed logging in Textual.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from rich.text import Text

class TextualApp(App[None]):

    def on_ready(self):

        self.log(Text.from_markup(
            "[bold][italic]Logger launched in on_ready method.[/italic] \n"
            "[green]Good Log message[/green] \n"
            "[blink red]Attention: something happened:"
        ))


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how to change the timeout for tooltips.
This is achieved by monkey-patching the _handle_tooltip_timer method
of the Screen class in Textual. Pyright / Mypy do not like this file.
I'm not even gonna bother trying to type hint it (I can assure you it works).

Recipe by Edward Jazzhands"""

from __future__ import annotations
import sys

from textual.app import App
from textual.widgets import Static, Footer
from textual.widget import Widget
from textual.css.query import NoMatches
from textual.containers import Container
from textual.widgets._tooltip import Tooltip
from textual.screen import Screen
from rich.console import RenderableType


def _handle_tooltip_timer(self, widget: Widget) -> None:

    try:
        tooltip = self.get_child_by_type(Tooltip)
    except NoMatches:
        pass
    else:
        tooltip_content: RenderableType | None = None
        for node in widget.ancestors_with_self:
            if not isinstance(node, Widget):
                break
            if node.tooltip is not None:
                tooltip_content = node.tooltip
                break

        if tooltip_content is None:
            tooltip.display = False
        else:
            tooltip.display = True
            tooltip.absolute_offset = self.app.mouse_position
            tooltip.update(tooltip_content)

            self.set_timer(2.0, lambda: setattr(tooltip, "display", False))


# MONKEY PATCH:
Screen._handle_tooltip_timer = _handle_tooltip_timer


class TextualApp(App[None]):

    DEFAULT_CSS = """
    #my_container { align: center middle; }
    #my_static { border: solid blue; width: auto;}
    """

    def compose(self):

        self.log("Composing the app")

        with Container(id="my_container"):
            mystatic = Static("Hello, Textual!", id="my_static")
            yield mystatic
            mystatic.tooltip = "This is a tooltip"

        yield Footer()


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This example demonstrates how the `push_screen_wait` method
(A wrapper around `push_screen` with the `wait_for_dismiss` argument
set to True) can be used to push a screen in the middle of a worker
that will cause the worker to temporarily pause what it is doing. This
is demonstrated in both a normal async worker and a threaded worker.

Recipe by NSPC911  
https://github.com/NSPC911"""

import sys
from typing import Any
import asyncio

from textual import on, work
from textual.app import App, ComposeResult
from textual.containers import Grid, Container
from textual.screen import ModalScreen
from textual.widgets import Button, Label, ProgressBar

class Dismissable(ModalScreen[None]):
    """Super simple screen that can be dismissed."""

    DEFAULT_CSS = """
    Dismissable {
        align: center middle
    }
    #dialog {
        grid-size: 1;
        grid-gutter: 1 2;
        grid-rows: 1fr 3;
        padding: 1 3;
        width: 50vw;
        max-height: 13;
        border: round $primary-lighten-3;
        column-span: 3
    }
    #message {
        height: 1fr;
        width: 1fr;
        content-align: center middle
    }
    Container {
        align: center middle
    }
    Button {
        width: 50%
    }
    """

    def __init__(self, message: str, **kwargs: Any):
        super().__init__(**kwargs)
        self.message = message

    def compose(self) -> ComposeResult:
        with Grid(id="dialog"):
            yield Label(self.message, id="message")
            with Container():
                yield Button("Ok", variant="primary", id="ok")

    def on_mount(self) -> None:
        self.query_one("#ok").focus()

    @on(Button.Pressed, "#ok")
    def on_button_pressed(self) -> None:
        """Handle button presses."""
        self.dismiss()


class TextualApp(App[None]):
    
    def compose(self) -> ComposeResult:
        yield Container()
        yield Button("test in normal worker", id="test")
        yield Button("test in threaded worker", id="test_threaded")
        
    @on(Button.Pressed, "#test")
    @work
    async def if_button_pressed(self, event: Button.Pressed) -> None:
        
        progress = ProgressBar(total=10)
        self.mount(progress)
        while progress.percentage != 1:
            progress.advance()
            await asyncio.sleep(0.5)
            if progress.percentage == 0.5:
                await self.push_screen_wait(Dismissable("hi"))
                
    @on(Button.Pressed, "#test_threaded")
    @work(thread=True)
    def if_button_pressed_thread(self) -> None:
        
        progress = ProgressBar(total=10)
        self.call_from_thread(self.mount, progress)
        while progress.percentage != 1:
            self.call_from_thread(progress.advance)
            self.call_from_thread(asyncio.sleep, 0.5)
            if progress.percentage == 0.5:
                self.call_from_thread(self.push_screen_wait, Dismissable("hi"))


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates how to create a TabbedContent widget with
tabs that cannot be clicked or interacted with by the user.
They can only be changed programmatically, which is demonstrated
by pressing the "Enter" key to switch to the next tab.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App, ComposeResult
from textual.widgets import Placeholder, Footer, TabbedContent, TabPane
from textual.containers import Container

class TextualApp(App[None]):    

    CSS = """
    Tabs {
        &:disabled {
            .underline--bar {
                background: $foreground 30%;
            }
            & .-active {
                color: $block-cursor-foreground;
                background: $block-cursor-background;
            }
        }
    }    
    """

    BINDINGS = [
        ("enter", "next_tab", "Next Tab"),
    ]

    current_tab = 1

    def compose(self) -> ComposeResult:
        with Container(id="main_container"):
            with TabbedContent() as TB:
                TB.disabled = True
                with TabPane("Tab 1", id="tab1"):
                    yield Placeholder("Placeholder for Tab 1")
                with TabPane("Tab 2", id="tab2"):
                    yield Placeholder("Placeholder for Tab 2")
                with TabPane("Tab 3", id="tab3"):
                    yield Placeholder("Placeholder for Tab 3")
        yield Footer()

    def action_next_tab(self) -> None:
        next_tab = (self.current_tab + 1) if self.current_tab < 3 else 1
        self.current_tab = next_tab
        self.query_one(TabbedContent).active = f"tab{next_tab}"


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates how to createa a "shaking" effect on a button.

Recipe by NSPC911"""


import sys
from textual import work
from textual.app import App, ComposeResult
from textual.widgets import Button

from asyncio import sleep

class TextualApp(App[None]):
    CSS = """
    Button {
        margin-left: 10
    }
    """
    def compose(self) -> ComposeResult:
        yield Button("Shake me!")
    @work
    async def on_button_pressed(self, event: Button.Pressed):
        self.query_one(Button).styles.margin = (0,0,0,11)
        await sleep(0.1)
        self.query_one(Button).styles.margin = (0,0,0,10)
        await sleep(0.1)
        self.query_one(Button).styles.margin = (0,0,0,11)
        await sleep(0.1)
        self.query_one(Button).styles.margin = (0,0,0,10)



if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This example shows how to use the rich library to display a spinner in Textual.
The spinner object changes its rendering internally, but Textual needs to be manually updated
to reflect that change. This is done by using `set_interval` to call the `update_spinner` method.

Recipe by Edward Jazzhands"""

from __future__ import annotations
import sys
from typing import Any, TYPE_CHECKING
if TYPE_CHECKING:
	from rich.console import RenderableType
from rich.spinner import Spinner


from textual.app import App
from textual.widgets import Static

class SpinnerWidget(Static):

    def __init__(self, spinner: str, text: RenderableType, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self._spinner = Spinner(spinner, text)  

    def on_mount(self) -> None:
        self.set_interval(0.02, self.update_spinner)

    def update_spinner(self) -> None:
        self.update(self._spinner)

class TextualApp(App[None]):

    CSS = """
        SpinnerWidget {width: 1fr; height: 1fr; content-align: center middle;}
    """

    def compose(self):

        yield SpinnerWidget("line", "Loading...")
        # A few common options:
        # arc, arrow, bouncingBall, boxBounce, dots, dots2 to dots12, line
        # python -m rich.spinner to see all options


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how a widget can be initialized with a loading state
and then updated after a delay (simulating the time taken to boot up the app).

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Static, Footer
from textual.containers import Container

class TextualApp(App[None]):

    DEFAULT_CSS = """
    #my_container { align: center middle; }
    #my_static { width: 40; height: 15; border: solid $primary; }
    """
    
    def compose(self):

        self.my_static = Static("Hello, Textual! great day eh", id="my_static")
        self.my_static.loading = True
        with Container(id="my_container"):
            yield self.my_static
        yield Footer()

    def on_ready(self):
        self.set_timer(2, self.finished_loading)

    def finished_loading(self):
        self.my_static.loading = False


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This example demonstrates a progress bar with a cool bouncing
animation effect.

Recipe by David Fokkema
https://github.com/davidfokkema"""

import sys
import time

from textual.app import App, ComposeResult
from textual.widgets import ProgressBar


class ElapsedTimeProgressBar(ProgressBar):
    _start_time: float = 0

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._start_time = time.monotonic()

    def update(self, *args, **kwargs) -> None:
        super().update(*args, **kwargs)
        if self.total is None:
            self._display_eta = int(time.monotonic() - self._start_time)


class TextualApp(App[None]):
    def compose(self) -> ComposeResult:
        yield ElapsedTimeProgressBar()
        yield ProgressBar()

    def on_mount(self) -> None:
        self.progress_timer = self.set_interval(1 / 10, self.make_progress, pause=True)
        self.set_timer(5.0, self.start_progress)

    def start_progress(self) -> None:
        for bar in self.query(ProgressBar):
            bar.update(total=100)
        self.progress_timer.resume()

    def make_progress(self) -> None:
        for bar in self.query(ProgressBar):
            bar.advance(1)


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how Textual can detect the color system
in use and display a message accordingly.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Static, Footer
from textual.containers import Container

class TextualApp(App[None]):

    DEFAULT_CSS = """
    Screen { width: 1fr; height: 1fr;}
    #my_static { border: solid blue; width: auto;}
    """
    
    def compose(self):

        with Container(id="my_container"):
            yield Static("Hello, Textual!", id="my_static")

        yield Footer()

    def on_ready(self):

        color_system = self.app.console.color_system

        if color_system == "256":
            self.notify("256 colors")
        elif color_system == "truecolor":
            self.notify("True color")
        elif color_system == "standard":
            self.notify("Standard color")
        else:
            self.notify("Unknown color system")

        # possibly set os.environ afterwards??
        # import os
        # os.environ["COLORTERM"] = "truecolor"


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates setting timers inside of workers.
A timer schedules a job to run on the event loop, and is independent
of the worker that created it. This demonstrates how in both cases
the timer callback runs after the worker has completed, as well as
how the thread worker must use call_from_thread to schedule timers.

Recipe by Edward Jazzhands
"""
from __future__ import annotations
import sys
import asyncio
import time
from typing import cast
from functools import partial

from textual import work, on
from textual.worker import Worker, get_current_worker  # type: ignore  ("Partially Unknown")
from textual.app import App, ComposeResult
from textual.containers import Horizontal, Vertical
from textual.widgets import RichLog, Button


class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    #log_container {
        width: 70%;
        min-width: 60;
        height: 90%;
        RichLog { border: heavy $primary; }
    }
    .buttonrow { 
        height: auto;
        Button { margin: 0 2; }
    }
    """

    def compose(self) -> ComposeResult:

        with Vertical(id="log_container"):
            with RichLog(id="log1", markup=True) as rlog:
                rlog.border_title = "Non-Threaded Worker Log"
                rlog.can_focus = False
            with Horizontal(classes="buttonrow"):
                yield Button("Start normal worker ", id="start_normal_worker")
                yield Button("Start thread worker", id="start_thread_worker")
                

    def update_log(self, message: str) -> None:

        log_widget = self.query_one(RichLog)
        log_widget.write(message)

    async def on_button_pressed(self, event: Button.Pressed) -> None:

        if event.button.id == "start_normal_worker":
            self.normal_worker()
        elif event.button.id == "start_thread_worker":
            self.threaded_worker()

    def worker_timer(self, set_by_worker: Worker[None], log_num: int) -> None:

        self.update_log(
            "[yellow]Timer callback[/yellow]: \n"
            f"[red]Set by[/red]: {set_by_worker.name} \n"
            f"[red]State[/red]: {set_by_worker.state} \n"
            f"[red]Is finished?[/red]: {set_by_worker.is_finished} \n"
        )

    @work
    async def normal_worker(self) -> None:

        current_worker = cast(Worker[None], get_current_worker())
        self.set_timer(2, partial(self.worker_timer, current_worker, 1))
        await asyncio.sleep(1)

    @work(thread=True)
    def threaded_worker(self) -> None:

        current_worker = cast(Worker[None], get_current_worker())

        # Threaded workers must use call_from_thread to schedule timers.
        # Calling set_timer by itself would raise an error.
        self.app.call_from_thread(
            self.set_timer,
            2, partial(self.worker_timer, current_worker, 2)
        )
        time.sleep(1)

    @on(Worker.StateChanged)
    def _worker_state_changed(self, event: Worker.StateChanged) -> None:

        worker = event.worker   # type: ignore  ("Unknown worker type")
        self.update_log(f"{worker.name} is {worker.state}")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how you can utilize the `disabled` state of a container
to make all of its widgets and contets greyed out and unresponsive.
A container greys out all of its children when it is disabled. This can be useful
for making sections of your UI that can be temporarily disabled. You can select
a container with your mouse and press 'd' to disable it.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Static, Footer, Button, Checkbox, Input
from textual.containers import Container


class SpecialContainer(Container):

    BINDINGS = [("d", "select", "Disable this container")]

    def compose(self):
        self.can_focus = True
        yield Static("Hello, Textual! great day eh")
        yield Button("A Button")
        yield Checkbox("A Checkbox")
        yield Input(placeholder="An Input")

    def action_select(self):
        self.notify(f"Disabled {self.id}")
        self.disabled = True


class TextualApp(App[None]):

    DEFAULT_CSS = """
    #main_container { 
        align: center middle;
        layout: grid;
        grid-size: 3 2;
    }
    .inner-container {
        border: solid $primary;
        &:focus { border: solid $accent; }
    }
    """
    
    def compose(self):

        with Container(id="main_container"):
            for i in range(6):
                yield SpecialContainer(id=f"container_{i}", classes="inner-container")
        yield Footer()


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates how Textual can animate multiple widgets at once.
I'm honestly not sure exactly of how it handles it internally, but it
seems to work fine when you simply run the animate method multiple times
in a row. It will make them all run simultaneously.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Static, Button
from textual.containers import Container
from textual.geometry import Offset

class TextualApp(App[None]):

    CSS = """
    #my_container { align: center middle; border: solid red;}
    #my_static1, #my_static2 { border: solid blue; width: auto;}
    """
    duration = 1.0

    def compose(self):

        with Container(id="my_container"):

            self.static1 = Static("Static 1", id="my_static1")
            self.static2 = Static("Static 2", id="my_static2")
            yield self.static1
            yield self.static2
            yield Button("press me")

    def on_button_pressed(self):
        self.static1.animate(
            "offset",
            Offset(20, 0),
            duration=self.duration,
        ) 
        self.static2.animate(
            "offset",
            Offset(20, 0),
            duration=self.duration,
        )    
        # Note how it uses `self.styles.animate` here instead of `self.animate`:
        self.static2.styles.animate(
            "opacity",
            0.0,
            duration=self.duration,
        )                    


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates how to use the rich traceback feature
in a Textual application to log errors to a file and display them
in HTML format. The application includes a button that, when pressed,
will intentionally cause an error, triggering the logging mechanism.

Recipe by Edward Jazzhands"""

import sys
from datetime import datetime
from textual.app import App
from textual.widgets import Static, Button
from textual.containers import Container  
from rich.console import Console
from rich.traceback import Traceback


class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    #my_static { border: solid blue; width: auto;}
    """

    def compose(self):

        with Container(id="my_container"):
            yield Static("Hello, Textual!", id="my_static")
            yield Button("Press me to cause a bug", classes="some-tcss-class")

    def on_button_pressed(self):
        self.query_one("#non_existent_widget")  # This will raise an error

    def _handle_exception(self, error: Exception) -> None:

        self.record_log_files(error)
        super()._handle_exception(error)


    def record_log_files(self, e: Exception):

        with open("error.txt", "w") as log_txt_file:
            console = Console(file=log_txt_file, record=True, width=100)
            console.print(f"TextualDon Error Report: {datetime.now()}\n")
            traceback = Traceback.from_exception(
                type(e),
                e,
                e.__traceback__,
                show_locals=True
            )
            console.print(traceback)            
            console.save_html("error.html")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates ?
Example by Edward Jazzhands, 2025"""

from textual import on
from textual.app import App, ComposeResult
from textual.widgets import Button
from textual.screen import Screen
from textual.message import Message

class MainScreen(Screen[None]):

    class EventFoo(Message):
        pass

    def compose(self) -> ComposeResult:
        yield Button("Press Me", id="main_button")

    def on_button_pressed(self) -> None:
        self.post_message(self.EventFoo())

class TuiApp(App[None]):

    SCREENS = {"main": MainScreen}

    def on_mount(self) -> None:
        self.push_screen("main")

    @on(MainScreen.EventFoo)
    def workfoo(self) -> None:

        main_screen = self.get_screen("main", MainScreen)
        main_screen.query_one("#main_button", Button).label = "Success"
        self.notify(f"EventFoo received from {main_screen}")


if __name__ == "__main__":
    app = TuiApp()
    app.run()


"""This script demonstrates what order that events are fired in Textual.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Footer
class TextualApp(App):

    def counter(self) -> int:
        self._counter += 1
        return self._counter      

    def __init__(self):         # This is the very first thing to run.
        super().__init__()
        self.foo = "foo"
        self._counter = 0
        self.log("This log statement won't work. The logger isn't set up yet")

    def on_load(self, event):
        self.log(f"on_load: {self.counter()} | foo: {self.foo}")

    def compose(self):
        self.log(f"compose: {self.counter()})")          #2

        yield Footer()

    def on_resize(self, event):
        self.log(f"on_resize: {self.counter()})")     #3 and 5 

    def on_mount(self, event):
        self.log(f"on_mount: {self.counter()})")      #4

    def on_ready(self, event):
        self.log(f"on_ready: {self.counter()})")      #6


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""Textual Cookbook Recipe Runner
================================

# ~ Type Checking (Pyright and MyPy) - Strict Mode
# ~ Linting - Ruff
# ~ Formatting - Black - max 110 characters / line
"""

# python standard lib
from __future__ import annotations
from typing import TypedDict, Any
import subprocess
import sys
import re
from pathlib import Path
from importlib import resources
# from importlib.abc import Traversable
# import importlib
from enum import Enum

# Textual imports
from textual import on, work, events
from textual.app import App, ComposeResult
from textual.widget import Widget
from textual.widgets import Static, DataTable, Button, Markdown, TextArea
from textual.widgets.data_table import ColumnKey, Column
from textual.screen import Screen, ModalScreen
from textual.message import Message
from textual.containers import Horizontal, Vertical
from textual.binding import Binding
from textual.geometry import clamp
from rich.text import Text

# Textual library imports
from textual_pyfiglet import FigletWidget

class RecipeData(TypedDict):
    name: str
    category: str
    author: str
    description: str
    text_blob: str


class SortableText(Text):

    def __lt__(self, other: object) -> bool:
        if isinstance(other, str):
            return self.plain < other
        if isinstance(other, Text):
            return self.plain < other.plain
        return NotImplemented


class DescriptionScreen(ModalScreen[bool]):

    BINDINGS = [
        Binding("escape", "close_screen", description="Close the help window."),
        Binding("left", "app.focus_previous", description="Focus Previous"),
        Binding("right", "app.focus_next", description="Focus Next"),
    ]

    def __init__(self, description: str) -> None:
        super().__init__()
        self.description = description

    def compose(self) -> ComposeResult:

        with Vertical(classes="description_container"):
            yield Markdown(self.description)
            with Horizontal(classes="button_container modal"):
                yield Button("Run", id="run_button_modal", compact=True)
                yield Button("Back", id="back_button", compact=True)

    @on(Button.Pressed, "#run_button_modal")
    def run_button_pressed(self) -> None:
        self.dismiss(True)

    @on(Button.Pressed, "#back_button")
    def back_button_pressed(self) -> None:
        self.dismiss(False)

    def on_click(self, event: events.Click) -> None:
        
        if isinstance(event.control, DescriptionScreen):
            self.dismiss(False)

    def action_close_screen(self) -> None:
        self.dismiss()


class ErrorScreen(ModalScreen[bool]):

    BINDINGS = [
        Binding("escape", "close_screen", description="Close the help window."),
    ]

    def __init__(self, error: subprocess.CalledProcessError) -> None:
        super().__init__()
        self.error = error

    def compose(self) -> ComposeResult:

        with Vertical(classes="description_container"):
            yield Static(
                "[$accent]Recipe failed to run[/$accent] \n\n"
                "Close the cookbook to view the error output.\n",
                markup=True
            )
            with Horizontal(classes="button_container modal"):
                yield Button("Close", id="close_button", compact=True)

    @on(Button.Pressed, "#close_button")
    def close_button_pressed(self) -> None:
        self.dismiss(False)

    def on_click(self, event: events.Click) -> None:
        
        if isinstance(event.control, ErrorScreen):
            self.dismiss(False)

    def action_close_screen(self) -> None:
        self.dismiss()


class ResizeBar(Static):

    DEFAULT_CSS = """
    ResizeBar {
        width: 1;
        height: 1fr;
        border-left: outer $panel;
        &:hover { border-left: outer $primary-darken-2; }
        &.pressed {   border-left: outer $primary-lighten-1; }
    }
    """

    def __init__(self, parent: Widget, **kwargs: Any) -> None:
        super().__init__(**kwargs)
        self.connected_container = parent
        self.min_width = 25    # adjust as needed
        self.max_width = 80
        self.tooltip = "<-- Drag to resize -->"

    def on_mouse_move(self, event: events.MouseMove) -> None:

        # App.mouse_captured refers to the widget that is currently capturing mouse events.
        if self.app.mouse_captured == self:

            total_delta = event.screen_offset - self.position_on_down
            new_size = self.size_on_down - total_delta
        
            self.connected_container.styles.width = clamp(new_size.width, self.min_width, self.max_width)

    def on_mouse_down(self, event: events.MouseDown) -> None:

        self.max_width = self.app.screen.size.width - 10
        self.position_on_down = event.screen_offset
        self.size_on_down = self.connected_container.size

        self.add_class("pressed")    # this requires a "pressed" class to exist
        self.capture_mouse()

    def on_mouse_up(self) -> None:

        self.remove_class("pressed")
        self.release_mouse()


class CodeContainer(Horizontal):

    def compose(self) -> ComposeResult:
        yield ResizeBar(self)
        with Vertical():
            with Horizontal():
                yield Static(id="current_recipe")
                yield Static(id="current_category")
                with Button("Run", id="run_button", compact=True) as butt:
                    butt.can_focus = False
            yield TextArea(
                "Select a recipe \n"
                "Hint: You can resize this area by dragging \n"
                "<-- this Resize bar with your mouse.",
                # language="markdown",
                read_only=True,
                show_line_numbers=True,
                soft_wrap=False,
                tab_behavior="indent"
            )

    def update(self, recipe_data: RecipeData):
        self.query_one("#current_recipe", Static).update(f"📄 {recipe_data['name']}.py  │")
        self.query_one("#current_category", Static).update(f"📁 {recipe_data['category']}")
        text_area = self.query_one(TextArea)
        text_area.language = "python"
        text_area.text = recipe_data['text_blob']


class CustomDataTable(DataTable[Any]):

    class TableInitialized(Message):
        pass

    class SortingStatus(Enum):
        UNSORTED = 0  #     [-] unsorted
        ASCENDING = 1  #    [↑] ascending (reverse = True)
        DESCENDING = 2  #   [↓] descending (reverse = False)


    def __init__(
        self,
        recipe_data_dict: dict[str, RecipeData], 
    ) -> None:
        super().__init__(zebra_stripes=True, show_cursor=True, cursor_type="row")
        self.recipe_data_dict = recipe_data_dict
        self.initialized = False

        self.sort_status: dict[str, CustomDataTable.SortingStatus] = {
            "name": CustomDataTable.SortingStatus.UNSORTED,
            "category": CustomDataTable.SortingStatus.UNSORTED,
            "author": CustomDataTable.SortingStatus.UNSORTED,
        }

        for key in self.sort_status:
            self.add_column(f"{key} [dark_orange]-[/]", key=key)

    def on_mount(self) -> None:

        for recipe_data in self.recipe_data_dict.values():

            self.add_row(
                SortableText(recipe_data["name"], overflow="ellipsis"),
                SortableText(recipe_data["category"], overflow="ellipsis"),
                SortableText(recipe_data["author"], overflow="ellipsis"),
            )        

    @on(DataTable.HeaderSelected)
    def header_selected(self, event: DataTable.HeaderSelected) -> None:

        column = self.columns[event.column_key]
        self.sort_column(column, column.key)


    def sort_column(self, column: Column, column_key: ColumnKey) -> None:

        if column_key.value is None:
            raise ValueError("Tried to sort a column with no key.")
        if column_key.value not in self.sort_status:
            raise ValueError(
                f"Unknown column key: {column_key.value}. "
                "This should never happen, please report this issue."
            )
        value = column_key.value

        # if its currently unsorted, that means the user is switching columns
        # to sort. Reset all columns to unsorted.
        if self.sort_status[value] == self.SortingStatus.UNSORTED:
            for key in self.sort_status:
                self.sort_status[key] = self.SortingStatus.UNSORTED
                col_index = self.get_column_index(key)
                col = self.ordered_columns[col_index]
                col.label = Text.from_markup(f"{key} [dark_orange]-[/]")

            # Now set chosen column to ascending:
            self.sort_status[value] = self.SortingStatus.ASCENDING
            self.sort(column_key, reverse=True)
            column.label = Text.from_markup(f"{value} [dark_orange]↑[/]")

        # For the other two conditions, we just toggle ascending/descending
        elif self.sort_status[value] == self.SortingStatus.ASCENDING:
            self.sort_status[value] = self.SortingStatus.DESCENDING
            self.sort(value, reverse=False)
            column.label = Text.from_markup(f"{value} [dark_orange]↓[/]")
        elif self.sort_status[value] == self.SortingStatus.DESCENDING:
            self.sort_status[value] = self.SortingStatus.ASCENDING
            self.sort(column_key, reverse=True)
            column.label = Text.from_markup(f"{value} [dark_orange]↑[/]")
        else:
            raise ValueError(
                f"Sort status for {value} is '{self.sort_status[value]}', did not meet any expected values."
            )


class TableScreen(Screen[None]):

    BINDINGS = [
        Binding("1", "sort_column(0)", "Sort Column 1"),
        Binding("2", "sort_column(1)", "Sort Column 2"),
        Binding("3", "sort_column(2)", "Sort Column 3"),
        Binding("d", "show_description", "Show Description"),
    ]

    def __init__(self, recipe_data_dict: dict[str, RecipeData]) -> None:
        super().__init__()
        self.recipe_data_dict = recipe_data_dict
        self.selected_recipe = None

    def compose(self) -> ComposeResult:

        with Vertical(id="main_container"):
            with Vertical(id="header_container"):
                yield FigletWidget(
                    "TeXTuaL cOOkBOOk",
                    font="double_blocky",
                    colors=["red", "red", "magenta"],
                    horizontal=True,
                    animate=True,
                    fps=30,
                )
                yield Static(
                    "Select once to view code, select again to run",
                    id="header_text",
                )
            with Vertical(id="table_container"):
                self.table = CustomDataTable(self.recipe_data_dict)
                yield self.table
            with Vertical(id="bottom_container"):
                yield Static(
                    "[$accent]Up/Down[/] Navigate │ "
                    "[$accent]Enter[/]/[$accent]Click[/] Select │ "
                    "[$accent]1-3[/] Sort │ "
                    "[$accent]d[/] Description",
                    id="controls_bar",
                )
                with Horizontal(classes="button_container"):
                    yield Button("Quit", id="quit_button", compact=True)
        yield CodeContainer()

    def action_sort_column(self, column_index: int) -> None:

        try:
            column = self.table.ordered_columns[column_index]
        except IndexError:
            return
        else:
            self.table.sort_column(column, column.key)

    @on(Button.Pressed, "#run_button")
    def run_button_pressed(self) -> None:

        if self.selected_recipe is None:
            self.notify("No recipe selected", timeout=3)
        else:
            category = self.recipe_data_dict[self.selected_recipe]["category"]
            self.run_recipe(self.selected_recipe, category)        

    @on(DataTable.RowSelected)
    @work
    async def recipe_selected(self, event: DataTable.RowSelected) -> None:

        row_data: list[str] = self.table.get_row(event.row_key)
        recipe_name = str(row_data[0])
        recipe_data = self.recipe_data_dict[recipe_name]
        category = self.recipe_data_dict[recipe_name]["category"]
        code_container = self.query_one(CodeContainer)

        if recipe_name == self.selected_recipe:
            self.run_recipe(recipe_name, category)
        else:
            self.selected_recipe = recipe_name
            code_container.update(recipe_data)

    @work
    async def action_show_description(self) -> None:

        cursor_index = self.table.cursor_row
        row_data: list[str] = self.table.get_row_at(cursor_index)
        recipe_name = str(row_data[0])
        description = self.recipe_data_dict[recipe_name]["description"]
        result = await self.app.push_screen(DescriptionScreen(description), wait_for_dismiss=True)
        if result:
            category = self.recipe_data_dict[recipe_name]["category"]
            self.run_recipe(recipe_name, category)

    def run_recipe(self, recipe_name: str, category: str) -> None:

        with self.app.suspend():

            cookbook = resources.files("textual_cookbook")
            recipe_traversable = cookbook.joinpath(f"recipes/{category}/{recipe_name}.py")

            # as_file gives a real filesystem Path (extracts to a temp file if inside a zip/wheel)
            with resources.as_file(recipe_traversable) as recipe_path:
                try:
                    subprocess.run([sys.executable, str(recipe_path)], check=True)
                except subprocess.CalledProcessError as e:
                    self.app.push_screen(ErrorScreen(e))
                except Exception as e:
                    raise e


    @on(Button.Pressed, "#quit_button")
    def quit_button_pressed(self) -> None:
        self.set_timer(0.2, self.app.exit)


class CookBookApp(App[None]):

    CSS_PATH = "styles.tcss"

    class WorkerFinished(Message):
        def __init__(self, recipe_data_dict: dict[str, RecipeData]) -> None:
            super().__init__()
            self.recipe_data_dict = recipe_data_dict

    def on_mount(self) -> None:
        self.run_worker(self.scan_for_recipes)

    async def scan_for_recipes(self) -> None:

        recipe_data_dict: dict[str, RecipeData] = {}
        
        # NEW: Start with Traversable then convert to Path for easier handling
        recipes_traversable = resources.files("textual_cookbook").joinpath("recipes")
        recipes_dir = Path(str(recipes_traversable))

        # Simple one-level scan - categories are subdirectories, no files in root
        for recipe_file in recipes_dir.rglob("*.py"):
            with open(recipe_file, "r", encoding="utf-8") as f:
                text_blob = f.read()
                author_match = re.search(r"Recipe by (.+)", text_blob)
                if author_match:
                    author = author_match.group(1).strip()
                    author = author.replace('"', "")
                else:
                    author = "Unknown"

            description_match = re.match(r'"""(.*?)"""', text_blob, re.DOTALL)
            if description_match:
                description = description_match.group(1).strip()
            else:
                description = "No description available."

            recipe_data: RecipeData = {
                "name": recipe_file.stem,
                "category": recipe_file.parent.name,
                "author": author,
                "description": description,
                "text_blob": text_blob
            }
            recipe_data_dict[recipe_file.stem] = recipe_data

        self.post_message(CookBookApp.WorkerFinished(recipe_data_dict))


    @on(WorkerFinished)
    async def worker_finished(self, message: WorkerFinished) -> None:

        await self.push_screen(TableScreen(message.recipe_data_dict))

def run_main() -> None:
    CookBookApp().run()

if __name__ == "__main__":
    run_main()


"""This file demonstrates how to create animating pop-up
context menus. The menu will pop up whereever you click
on the static banner in the center.
The context menu itself is actually on a Modal Screen. This
allows us to close it by clicking anywhere.

Recipe by Edward Jazzhands"""

# Python imports
from __future__ import annotations
from typing import TYPE_CHECKING
import sys

if TYPE_CHECKING:
    from textual.app import ComposeResult

# Textual and rich imports
from textual import work
from textual.app import App, ComposeResult
from textual.geometry import Offset
from textual.screen import ModalScreen
from textual.containers import Container
from textual import events
from textual.widgets import Static, Button


class ContextMenu(ModalScreen[None]):
    
    BINDINGS = [
        ("up", "app.focus_previous"),
        ("down", "app.focus_next"),
    ]

    CSS = """
    ContextMenu {
        background: $background 0%;
        align: left top;    /* This will set the starting coordinates to (0, 0) */
    }                       /* Which we need for the absolute offset to work */
    #menu_container {
        background: $surface;
        width: 10;
        border-left: wide $panel;
        border-right: wide $panel;        
        &.bottom { border-top: hkey $panel; }
        &.top { border-bottom: hkey $panel; }
        & > Button {
            min-width: 8;
            &:hover { background: $panel-lighten-2; }
            &:focus {
                background: $panel-lighten-2; 
                text-style: none;
            }
        }
    }
    """

    def __init__(self, menu_offset: Offset) -> None:
        super().__init__()
        self.menu_offset = menu_offset

    def compose(self) -> ComposeResult:

        with Container(id="menu_container"):
            yield Button("Menu 1", id="menu1", compact=True)
            yield Button("Menu 2", id="menu2", compact=True)
            yield Button("Menu 3", id="menu3", compact=True)
            yield Button("Menu 4", id="menu4", compact=True)

    def on_mount(self) -> None:

        menu = self.query_one("#menu_container")
        # automatically set the height to the number of buttons:
        height = len(menu.children)
        menu.styles.height = height
        
        # Subtracting the menu's height will make it appear above
        # the mouse instead of below it
        y_offset = self.menu_offset.y - height
        menu.offset = Offset(self.menu_offset.x, y_offset)

    def on_mouse_up(self) -> None:

        # This allows us to close it by clicking anywhere
        self.dismiss(None)

    async def on_button_pressed(self, event: Button.Pressed) -> None:

        if event.button.id == "menu1":
            self.notify("Menu 1")
        elif event.button.id == "menu2":
            self.notify("Menu 2")
        elif event.button.id == "menu3":
            self.notify("Menu 3")
        elif event.button.id == "menu4":
            self.notify("Menu 4")
            
        self.dismiss()

class MyStatic(Static):
    
    @work
    async def on_click(self, event: events.Click) -> None:
        menu_offset = event.screen_offset
        await self.app.push_screen_wait(
            ContextMenu(menu_offset=menu_offset)
        )

class TextualApp(App[None]):

    CSS = """
    #my_container { 
        align: center middle; 
        border: solid red;
    }
    MyStatic { 
        border: hkey blue;
        width: auto;
        &:hover { background: $boost; }
    }
    """

    def compose(self):

        with Container(id="my_container"):
            yield MyStatic(
                "Click on me anywhere. "
                "Notice how the menu pops up where you click. \n"
                "You can also navigate the menu with up/down keys or tab.",
                id="main_button"
            )
             

if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This example demonstrates making a progress bar with different colors
for different states: indeterminate, complete, incomplete, and error.

Recipe by NSPC911  
https://github.com/NSPC911"""

import sys
from textual.app import App, ComposeResult
from textual.widgets import ProgressBar

class TextualApp(App[None]):
    CSS = """
    .bar {
      color: $warning
    }
    .bar--indeterminate {
      color: $accent
    }
    .bar--complete {
      color: $success;
    }
    .error .bar--complete,
    .error .bar--bar {
      color: $error;
    }
    """
    def compose(self) -> ComposeResult:
        yield ProgressBar(total=None, id="indeterminate")
        yield ProgressBar(total=100, id="complete", classes="complete")
        yield ProgressBar(total=100, id="incomplete", classes="incomplete")
        yield ProgressBar(total=100, id="error", classes="error complete")
        yield ProgressBar(total=100, id="incompleteerror", classes="error incomplete")
        
    def on_mount(self) -> None:
        for widget in self.query("ProgressBar.incomplete"):
            widget.update(progress=50)
        for widget in self.query("ProgressBar.complete"):
            widget.update(progress=100)
            


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file shows how to change the color and style of the
title/header of a Collapsible widget in Textual.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Collapsible, Button


class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    Collapsible { 
        width: 50%; height: auto; 
        CollapsibleTitle {
            background: transparent;
            color: blue;    /* text color */
        }
    }
    """

    def compose(self):

        for _ in range(3):
            with Collapsible():
                for _ in range(3):
                    yield Button("Hello, Textual!")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how you cannot use the same widget in multiple places.
The second Static `widget2` will not be displayed twice because it is already
displayed in the first container.

Example by Edward Jazzhands, 2025"""

from textual.app import App
from textual.widgets import Static, Footer
from textual.containers import Container

class TextualApp(App[None]):

    DEFAULT_CSS = """
    #my_container {
        width: 1fr; height: 1fr;
        border: solid red;
        align: center middle; content-align: center middle;
    }
    Static { border: solid blue; width: auto;}
    """
    
    def compose(self):

        widget = Static("Hello, Textual!")
        widget2 = Static("Hello, Textual! 2")

        with Container(id="my_container"):
            yield widget2
        with Container(id="my_container2"):
            yield widget
            yield widget2

        yield Footer()


TextualApp().run()


"""This example demonstrates how to set up and read a dictionary on the app class
in order to share data between different components.

Recipe by Edward Jazzhands"""

import sys
from textual import on
from textual.app import App
from textual.widgets import Button
from textual.containers import Container

class MyContainer(Container):

    DEFAULT_CSS = """ MyContainer {border: panel $primary;} """

    def __init__(self, container_num:int):
        super().__init__()
        self.container_num = container_num
        self.border_title = f"MyContainer {container_num}"

    def compose(self):
        yield Button("Set app dict", id="button1")
        yield Button("Read app dict", id="button2")

    @on(Button.Pressed, "#button1")
    def button1(self):
        x = self.container_num
        my_dict = {
            f"key{x}": f"value{x}",
            f"key{x+1}": f"value{x+1}"
        }
        self.app.app_dict = my_dict          # type: ignore
        self.notify(f"App dict set by container {self.container_num}")

    @on(Button.Pressed, "#button2")
    def button2(self):
        self.notify(str(self.app.app_dict))  # type: ignore
        self.log(self.app.app_dict)          # type: ignore 


class TextualApp(App[None]):

    app_dict = {}

    def compose(self):
        yield MyContainer(1)
        yield MyContainer(3)


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)